
# q-expansion of newform 5520.2.a.bb, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5520
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [1], [-4], [-2], [4], [-6], [4], [1], [8], [-8], [-10], [6], [-6], [4], [-14], [-4], [6], [-14], [-10], [14], [8], [4], [0], [-8], [-8], [-4], [-8], [-14], [-10], [-2], [-16], [6], [20], [-10], [-20], [-2], [8], [8], [10], [4], [-2], [16], [-22], [-22], [-16], [20], [-14], [-8], [-22], [-6], [-26], [10], [2], [18], [-16], [12], [8], [-12], [8], [22], [26], [-4], [14], [-4], [-18], [-12], [-20], [-12], [-30], [14], [16], [12], [-18], [16], [24], [-14], [-28], [20], [-10], [30], [6], [-8], [32], [20], [28], [30], [16], [0], [10], [36], [-24], [10], [24], [-20], [0], [24], [0], [14], [10], [-28], [-2], [12], [-44], [24], [-34], [-12], [18], [-14], [-26], [-30], [-14], [18], [20], [8], [-24], [-34], [28], [-42], [6], [-22], [-34], [-26], [-12], [4], [-6], [34], [-46], [-32], [14], [4], [-32], [16], [34], [-6], [14], [42], [-50], [54], [-6], [12], [12], [-42], [48], [-38], [12], [-4], [6], [4], [36], [20], [-48], [-8], [-12], [-46], [36], [16], [-18], [0], [38], [20], [38], [-22], [-6], [10], [0], [-4], [4], [18], [-6], [10], [-10], [-16], [32], [20], [60], [-36], [28], [-8], [58], [50], [42], [-28], [18], [-16], [10], [-36], [-46], [6], [42], [50], [36], [-16], [8], [60], [14], [14], [-42], [-26], [-36], [-30], [28], [32], [2], [-6], [26], [-56], [-20], [-14], [4], [30], [48], [-36], [-28], [-18], [22], [-10], [50], [0], [62], [-58], [-48], [-30], [-24], [-20], [54], [70], [-6], [-4], [44], [40], [56], [32], [14], [-26], [0], [-26], [-54], [-36], [-38], [44], [-20], [-34], [-70], [-30], [-22], [38], [-42], [20], [48], [-78], [-40], [56], [46], [-42], [8], [42], [38], [46], [-14], [-68], [16], [-66], [74], [-18], [64], [-6], [0], [-14], [-18], [6], [30], [-24], [24], [62], [14], [-68], [10], [-50], [-6], [32], [32], [-40], [22], [48], [-50], [-4], [-6], [44], [30], [30], [-72], [-58], [10], [16], [-34], [40], [-86], [-20], [-76], [-52], [-34], [72], [44], [-52], [38], [-84], [82], [-84], [-88], [12], [70], [-8], [-76], [24], [74], [40], [58], [-48], [74], [-20], [-4], [-12], [-74], [-54], [58], [-36], [44], [0], [-66], [-82], [-2], [-68], [12], [4], [-28], [62], [-14], [62], [-32], [-16], [-22], [-8], [80], [78], [-36], [-2], [-32], [-60], [70], [4], [-78], [-40], [-8], [-34], [6], [44], [30], [-78], [-40], [26], [6], [80], [-62], [-36], [-50], [-42], [-44], [-58], [40], [-76], [0], [82], [88], [56], [56], [-36], [-44], [0], [22], [34], [-2], [2], [22], [56], [84], [-44], [52], [-54], [-48], [-2], [102], [-24], [-92], [-24], [28], [-80], [-44], [-32], [-88], [-34], [-94], [-32], [-18], [8], [-52], [50], [-22], [86], [52], [-86], [34], [-12], [-28], [52], [76], [-90], [-90], [-42], [16], [-14], [22], [-80], [36], [-14], [42], [48], [-70], [92], [34]]
